searchData={"items":[{"type":"module","title":"Hardhat","doc":"An opinionated, production-ready HTTP client for Elixir services.","ref":"Hardhat.html"},{"type":"module","title":"What's included - Hardhat","doc":"- [X] Connection pooling per-client module\n- [X] Integration with `telemetry` and `opentelemetry` instrumentation\n- [X] Circuit breaking for repeatedly failed requests\n- [X] Automatic retries for failed requests\n- [X] Timeout and `deadline` support","ref":"Hardhat.html#module-what-s-included"},{"type":"module","title":"Installation - Hardhat","doc":"Add `hardhat` to the dependencies in your `mix.exs`:\n\n```elixir\n  def deps do\n    [\n      {:hardhat, \"~> 1.0.0\"}\n    ]\n  end\n```","ref":"Hardhat.html#module-installation"},{"type":"module","title":"Getting started - Hardhat","doc":"`Hardhat` is designed to be easy for creating quick wrappers around HTTP APIs,\nbut includes many options for customization. To define a simple client, do something like the following:\n\n```elixir\n# Define a client module:\ndefmodule SocialMediaAPI do\n  use Hardhat\nend\n\n# Add it to your supervisor (required):\ndefmodule MyApp.Sup do\n  use Supervisor\n\n  def start_link(init_arg) do\n    Supervisor.start_link(__MODULE__, init_arg, name: __MODULE__)\n  end\n\n  @impl true\n  def init(_init_arg) do\n    children = [\n      SocialMediaAPI\n    ]\n\n    Supervisor.init(children, strategy: :one_for_one)\n  end\nend\n\n# Use your client to make requests:\nSocialMediaAPI.get(\"http://media-api.social/posts\")\n```\n\nAs mentioned in the example above, it is imperative for you to supervise the client module that includes the `use Hardhat` macro. Without starting the client under supervision, you will not be able to make requests. See [Connection pools](#module-connection-pools) below for more information.","ref":"Hardhat.html#module-getting-started"},{"type":"module","title":"General behavior - Hardhat","doc":"TODO: Contents of the default middleware, link to other sections\n\n`Hardhat` is built on top of `Tesla`, and uses `Finch` as the adapter. Because\n`Tesla` is the foundation, you are welcome to utilize publicly available\n`Tesla.Middleware` modules in your `Hardhat`-based client (with the exception\nthat [we recommend](#module-timeouts-and-deadlines) you use\n`Hardhat.Middleware.Timeout` instead of `Tesla.Middleware.Timeout`).\n\n```elixir\ndefmodule SomeJSONAPI do\n  use Hardhat\n\n  plug Tesla.Middleware.BaseUrl, \"https://my-json.api/\"\n  plug Tesla.Middleware.JSON\nend\n```\n\nIn addition to the adapter selection and default `Tesla` behavior,\n`use Hardhat` will inject the common functionality [listed above](#module-what-s-included) *after* any middleware that you supply via [`plug`](`Tesla.Builder.plug/2`). The current list is as follows:\n\n* `Hardhat.Middleware.DeadlinePropagation`\n* `Tesla.Middleware.Retry`\n* Either `Tesla.Middleware.Fuse` or `Hardhat.Middleware.Regulator` ([See below](#module-failure-detection)\n* `Tesla.Middleware.Telemetry`\n* `Tesla.Middleware.OpenTelemetry`\n* `Hardhat.Middleware.PathParams`","ref":"Hardhat.html#module-general-behavior"},{"type":"module","title":"Connection pools - Hardhat","doc":"TODO: Sizing the pool, `Finch` options","ref":"Hardhat.html#module-connection-pools"},{"type":"module","title":"Telemetry and tracing - Hardhat","doc":"TODO: `:telemetry` events, `OpenTelemetry` spans and events, propagation","ref":"Hardhat.html#module-telemetry-and-tracing"},{"type":"module","title":"Failure detection - Hardhat","doc":"TODO: `:fuse` mode and configuration, `Regulator` mode and configuration","ref":"Hardhat.html#module-failure-detection"},{"type":"module","title":"Retries - Hardhat","doc":"TODO: default options, note about interactions with failure detection","ref":"Hardhat.html#module-retries"},{"type":"module","title":"Timeouts and deadlines - Hardhat","doc":"TODO: Custom timeout middleware, `Deadline` support","ref":"Hardhat.html#module-timeouts-and-deadlines"},{"type":"module","title":"Testing - Hardhat","doc":"TODO: overriding the adapter to use a mock or bypass","ref":"Hardhat.html#module-testing"},{"type":"module","title":"Hardhat.Defaults","doc":"Contains default implementations of functions that can be overridden in\nclients that `use Hardhat`.","ref":"Hardhat.Defaults.html"},{"type":"function","title":"Hardhat.Defaults.deadline_propagation_opts/0","doc":"Default implementation of the options to `Hardhat.Middleware.DeadlinePropagation` that\nis included in the default middleware stack.\n\nThese defaults include:\n- `:header` - `\"deadline\"` is the name of the header added to the request","ref":"Hardhat.Defaults.html#deadline_propagation_opts/0"},{"type":"function","title":"Hardhat.Defaults.fuse_opts/1","doc":"Default implementation of the options to the `Tesla.Middleware.Fuse` that is\nincluded in the default middleware stack.\n\nTakes the client module name as an argument.\n\nThese defaults include:\n- `:opts` - The fuse will blow after 50 errors with 1 second, and reset after 2 seconds\n- `:keep_original_error` - The original error will be returned when a fuse first blows\n- `:should_melt` - The `should_melt/1` function defined in the client module is used\n   (by default this is `Hardhat.Defaults.should_melt/1`)\n- `:mode` - The fuse uses `:async_dirty` mode to check the failure rate, which may result\n  in some delay in blowing the fuse under high concurrency, but it will not serialize\n  checks to the fuse state through a single process\n\nSee `Tesla.Middleware.Fuse` for more information on how to configure the middleware.","ref":"Hardhat.Defaults.html#fuse_opts/1"},{"type":"function","title":"Hardhat.Defaults.pool_options/1","doc":"The default options for the connection pool that will be created when your\nclient is added to the supervision tree.\n\nThis creates a connection pool of size `10`.","ref":"Hardhat.Defaults.html#pool_options/1"},{"type":"function","title":"Hardhat.Defaults.regulator_opts/1","doc":"Default options for the `Regulator` middleware, which can be used as an\nalternative circuit-breaking strategy to `fuse`.\n\nThe defaults include:\n* `:min_limit` - The minimum concurrency limit (defaults to 5)\n* `:initial_limit` - The initial concurrency limit when the regulator is installed (deafults to 20)\n* `:max_limit` - The maximum concurrency limit (defaults to 200)\n* `:step_increase` - The number of tokens to add when regulator is increasing the concurrency limit (defaults to 10).\n* `:backoff_ratio` - Floating point value for how quickly to reduce the concurrency limit (defaults to 0.9)\n* `:target_avg_latency` - This is the average latency in milliseconds for the system regulator is protecting. If the average latency drifts above this value Regulator considers it an error and backs off. Defaults to 5.\n* `:should_regulate` - Whether to consider the result of the request as failed, defaults to `should_regulate/1`.","ref":"Hardhat.Defaults.html#regulator_opts/1"},{"type":"function","title":"Hardhat.Defaults.retry_opts/1","doc":"Default implementation of the options to `Tesla.Middleware.Retry` that is\nincluded in the default middleware stack.\n\nThese defaults include:\n- `:delay` - The base delay in milliseconds (positive integer, defaults to 50)\n- `:max_retries` - maximum number of retries (non-negative integer, defaults to 3)\n- `:max_delay` - maximum delay in milliseconds (positive integer, defaults to 300)\n- `:should_retry` - function to determine if request should be retried, defaults to `should_retry/1`\n- `:jitter_factor` - additive noise proportionality constant (float between 0 and 1, defaults to 0.2)","ref":"Hardhat.Defaults.html#retry_opts/1"},{"type":"function","title":"Hardhat.Defaults.should_melt/1","doc":"Default implementation of the \"melt test\" for circuit breaking. This\nfunction will cause the circuit breaker to record an error when the\nresult of a request is:\n\n* A TCP-level error, e.g. `{:error, :econnrefused}`\n* An HTTP status that indicates a server error or proxy-level error (>= 500)\n* An `429 Too Many Requests` HTTP status","ref":"Hardhat.Defaults.html#should_melt/1"},{"type":"function","title":"Hardhat.Defaults.should_regulate/1","doc":"Default implementation of the \"failure test\" for dynamic regulation. This\nfunction will cause the `Regulator` to record an error when the\nresult of a request is:\n\n* A TCP-level error, e.g. `{:error, :econnrefused}`\n* An HTTP status that indicates a server error or proxy-level error (>= 500)\n* An `429 Too Many Requests` HTTP status","ref":"Hardhat.Defaults.html#should_regulate/1"},{"type":"function","title":"Hardhat.Defaults.should_retry/1","doc":"Default implementation of the \"retry test\" for retries. This\nfunction will cause requests to be retried when the result of the\nrequest is:\n\n* A TCP-level error, e.g. `{:error, :econnrefused}`\n* An HTTP status that indicates a server error or proxy-level error (>= 500)\n* An `429 Too Many Requests` HTTP status\n\nIn the case where the circuit breaker has been triggered, requests will\nnot be retried.","ref":"Hardhat.Defaults.html#should_retry/1"},{"type":"module","title":"Hardhat.Middleware.DeadlinePropagation","doc":"Propagates `Deadline` information to the server being called via\nan HTTP header. _This middleware is part of the default stack_.\n\nExpects a `:header` option, which defaults to `\"deadline\"`. See `Hardhat.Defaults.deadline_propagation_opts/0`.","ref":"Hardhat.Middleware.DeadlinePropagation.html"},{"type":"module","title":"Hardhat.Middleware.PathParams","doc":"Use templated URLs with separate params. Unlike `Tesla.Middleware.PathParams`,\nwe ensure that all parameters are URL-safe. _This middleware is part of the\ndefault stack_.","ref":"Hardhat.Middleware.PathParams.html"},{"type":"module","title":"Hardhat.Middleware.Regulator","doc":"Limits concurrent requests to the server based on a [loss based dynamic limit algorithm](`Regulator.Limit.AIMD`).\nAdditively increases the concurrency limit when there are no errors and multiplicatively\ndecrements the limit when there are errors.\n\n_This middleware is part of the default stack when giving the `strategy: :regulator` option to `use Hardhat`._\n\nSee also:\n- Configuration options: `Hardhat.Defaults.regulator_opts/1`\n- Determining what amounts to an error: `Hardhat.Defaults.should_regulate/1`","ref":"Hardhat.Middleware.Regulator.html"},{"type":"module","title":"Hardhat.Middleware.Timeout","doc":"Abort HTTP requests after the given timeout or the current `Deadline`,\nand emit `OpenTelemetry` trace events on timeouts.\n\nOptions:\n- `:timeout` - (required) timeout in milliseconds","ref":"Hardhat.Middleware.Timeout.html"},{"type":"module","title":"OpenTelemetry - Hardhat.Middleware.Timeout","doc":"Implementing a timeout necessitates moving the request into a new process,\nand then waiting on that new process's completion (or aborting after the timeout).\nSince `OpenTelemetry` tracing context is stored in the process dictionary, that\ncontext must be explicitly propagated to the new process. This middleware uses\n`OpentelemetryProcessPropagator` for this purpose.\n\nIn the event of a timeout result in this middleware, a new `timeout_exceeded` event\nwill be added to the trace. The event will include these attributes:\n\n- `module` - the client module that includes this middleware\n- `timeout` - the duration that was exceeded","ref":"Hardhat.Middleware.Timeout.html#module-opentelemetry"},{"type":"module","title":"Deadline - Hardhat.Middleware.Timeout","doc":"When the caller has set a `Deadline` for the current process, that limit will be\nrespected by this middleware. The effective timeout chosen will be the **lesser** of the time\nremaining on the current deadline and the duration given in the `:timeout` option.","ref":"Hardhat.Middleware.Timeout.html#module-deadline"}],"content_type":"text/markdown"}